
# q-expansion of newform 325.6.b.b, downloaded from the LMFDB on 04 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 325
weight = 6
poly_data = [16, 0, 9, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 1, 0, 0], 1], [[0, 5, 0, 1], 4], [[5, 0, 1, 0], 1]]

hecke_ring_character_values = [[27, [-1, 0, 0, 0]], [301, [1, 0, 0, 0]]]
aps_data = [[0, 1, 3, 0], [0, 6, -11, 0], [0, 0, 0, 0], [0, -70, -17, 0], [-230, 0, 0, 84], [0, 0, -169, 0], [0, -128, 1251, 0], [142, 0, 0, 28], [0, -1416, -2020, 0], [382, 0, 0, 48], [3636, 0, 0, 448], [0, -1840, 7509, 0], [2944, 0, 0, 1952], [0, -4718, -1149, 0], [0, -9670, -9821, 0], [0, 6816, -18452, 0], [15134, 0, 0, 8668], [5640, 0, 0, -9296], [0, -196, 34866, 0], [31865, 0, 0, 3666], [0, -18560, 27926, 0], [22780, 0, 0, 9736], [0, -17920, -46816, 0], [-70006, 0, 0, 23504], [0, 58720, 46738, 0], [50788, 0, 0, 34496], [0, 51472, 18136, 0], [0, -39440, 88436, 0], [84533, 0, 0, -8320], [0, 23648, 77506, 0], [0, 120592, 60560, 0], [-54035, 0, 0, 61290], [0, -56384, 124084, 0], [-170109, 0, 0, 6814], [265250, 0, 0, -131872], [397989, 0, 0, 35538], [0, -28560, -44870, 0], [0, -1496, -223332, 0], [0, -62056, -112624, 0], [0, -163952, -209164, 0], [276835, 0, 0, 76414], [-175168, 0, 0, 39456], [-378754, 0, 0, 60364], [0, -316880, -167440, 0], [0, -103056, -505537, 0], [102510, 0, 0, -292100], [-487707, 0, 0, -148470], [0, -62790, 39707, 0], [0, 231472, -473224, 0], [-348033, 0, 0, -313016], [0, 302520, 158069, 0], [-701555, 0, 0, 338538], [307186, 0, 0, 149824], [-1335912, 0, 0, 370488], [0, 62992, -356625, 0], [0, -299976, -722100, 0], [-616248, 0, 0, -91632], [-190905, 0, 0, 58550], [0, -279504, -564076, 0], [-1790506, 0, 0, 21296], [0, -546096, -420876, 0], [0, -677056, 1055215, 0], [0, 52252, 319714, 0], [-1581098, 0, 0, -134756], [0, -218136, 1985823, 0], [0, 956160, 501666, 0], [1486580, 0, 0, -844216], [0, -136024, 2347929, 0], [0, -381730, 3168657, 0], [-534999, 0, 0, -581144], [0, 739344, 893572, 0], [2022768, 0, 0, -165432], [0, 1831308, 1004202, 0], [0, -957840, -71300, 0], [2123552, 0, 0, -2075280], [0, 608874, -550357, 0], [116494, 0, 0, -206880], [0, -1055232, 1750726, 0], [431912, 0, 0, 703376], [3488556, 0, 0, 379824], [1247097, 0, 0, -2137382], [-140229, 0, 0, 1662328], [3727851, 0, 0, -1005098], [0, 329192, -547785, 0], [-1458702, 0, 0, -2352060], [0, -111690, -3105871, 0], [-7548710, 0, 0, -158048], [0, 327968, 2839734, 0], [4687171, 0, 0, -352], [0, 1431864, 4411040, 0], [0, 641088, 2140316, 0], [941673, 0, 0, 2241290], [0, -4664, 4072064, 0], [-2764607, 0, 0, 257762], [-5940512, 0, 0, -10400], [0, -211972, -9683374, 0], [-3361490, 0, 0, 4620128], [1314255, 0, 0, -590216], [0, 2637872, 4153452, 0], [6016297, 0, 0, -2206896], [0, 2744858, -3665789, 0], [0, -3943328, 2260343, 0], [0, 337194, -15821, 0], [-1400383, 0, 0, -3262032], [1017643, 0, 0, -5938914], [0, 2152560, -2197106, 0], [0, -4329672, 180636, 0], [0, 2283552, -5480118, 0], [2230726, 0, 0, 41404], [-3870595, 0, 0, 5581744], [0, 3485100, -5321894, 0], [0, 1864000, 10546826, 0], [0, 4638544, -394544, 0], [1710724, 0, 0, 1442840], [-1456735, 0, 0, -3188998], [3413694, 0, 0, -7530176], [0, 7748068, 3624478, 0], [0, 8470244, 2590534, 0], [0, 2080928, 2062808, 0], [331372, 0, 0, -3996368], [-9705306, 0, 0, -1285728], [0, -975376, -11727227, 0], [0, -6335936, -12437148, 0], [0, -667032, -12963340, 0], [7523400, 0, 0, -4985872], [-16343468, 0, 0, -1973760], [2099978, 0, 0, -2519296], [7789538, 0, 0, -11683692], [0, -1685804, 7464654, 0], [0, -6902120, -14004627, 0], [-5273796, 0, 0, -710152], [0, 5219118, 11361809, 0], [10314808, 0, 0, 2457696], [0, 4085136, -7608180, 0], [12434598, 0, 0, 512832], [11800512, 0, 0, -642432], [0, 6950504, 3297507, 0], [0, 57856, 940608, 0], [0, 803840, 13078426, 0], [-12512543, 0, 0, -1167032], [30528492, 0, 0, -2319848], [-14096681, 0, 0, 4201640], [0, -4851516, 10663766, 0], [0, 4350060, -13688398, 0], [5194918, 0, 0, 5779984], [-3062864, 0, 0, -849880], [0, -1678840, -10663687, 0], [0, 759104, 7159346, 0], [28360412, 0, 0, 4837568], [0, -820882, -16103499, 0], [0, -941560, -1112497, 0], [-21383595, 0, 0, 1835680], [0, 5559302, 14957305, 0], [0, 7084464, -5806168, 0], [0, -18332058, -3652447, 0], [26925374, 0, 0, 5923020], [-10490744, 0, 0, 12662912], [9206836, 0, 0, 1711952], [0, 4463008, -37071030, 0], [25861081, 0, 0, -14244712], [0, 11952300, 5506754, 0], [0, 21492064, 9143271, 0], [0, 12369766, 24953113, 0], [17000755, 0, 0, 8547734], [0, 9953776, 11617726, 0], [0, -16141466, 5400913, 0], [-15446682, 0, 0, 404540], [0, -2105984, -28374886, 0]]
